@@ -278,6 +278,9 @@ DotPlotAtomic <- function(
 #'         facet_scales = "free_x")
 #' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
 #'         fill_by = "mpg", fill_cutoff = 18, split_by = "cyl")
+#' DotPlot(mtcars, x = "carb", y = "gear", size_by = "wt",
+#'         fill_by = "mpg", fill_cutoff = 18, split_by = "cyl",
+#'         palette = list("4" = "Set1", "6" = "Paired", "8" = "Reds"))
 #' # works as a scatter plot
 #' DotPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
 #'         fill_by = "mpg", fill_cutoff = 18, fill_cutoff_name = "Small mpgs")
@@ -304,6 +307,8 @@ DotPlot <- function(
         datas <- list(data)
         names(datas) <- "..."
     }
+    palette <- check_palette(palette, names(datas))
+    palcolor <- check_palcolor(palcolor, names(datas))
 
     plots <- lapply(
         names(datas), function(nm) {
@@ -316,7 +321,7 @@ DotPlot <- function(
             DotPlotAtomic(datas[[nm]],
                 x = x, y = y, x_sep = x_sep, y_sep = y_sep, flip = flip, bg_direction = bg_direction,
                 size_by = size_by, fill_by = fill_by, fill_cutoff = fill_cutoff, fill_reverse = fill_reverse,
-                theme = theme, theme_args = theme_args, palette = palette, palcolor = palcolor, alpha = alpha,
+                theme = theme, theme_args = theme_args, palette = palette[[nm]], palcolor = palcolor[[nm]], alpha = alpha,
                 facet_by = facet_by, facet_scales = facet_scales, facet_ncol = facet_ncol, facet_nrow = facet_nrow, facet_byrow = facet_byrow,
                 x_text_angle = x_text_angle, size_name = size_name, fill_name = fill_name, fill_cutoff_name = fill_cutoff_name,
                 add_bg = add_bg, bg_palette = bg_palette, bg_palcolor = bg_palcolor, bg_alpha = bg_alpha,
@@ -344,6 +349,8 @@ DotPlot <- function(
 #' LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
 #'              fill_by = "mpg", fill_cutoff = 18, facet_by = "cyl",
 #'              facet_scales = "free_y")
+#' LollipopPlot(mtcars, x = "qsec", y = "drat", size_by = "wt",
+#'              split_by = "vs", palette = list("0" = "Reds", "1" = "Blues"))
 LollipopPlot <- function(
     data, x, y, y_sep = NULL, flip = FALSE,
     split_by = NULL, split_by_sep = "_", size_name = NULL, fill_name = NULL, fill_cutoff_name = NULL,
@@ -366,6 +373,8 @@ LollipopPlot <- function(
         datas <- list(data)
         names(datas) <- "..."
     }
+    palette <- check_palette(palette, names(datas))
+    palcolor <- check_palcolor(palcolor, names(datas))
 
     plots <- lapply(
         names(datas), function(nm) {
@@ -378,7 +387,7 @@ LollipopPlot <- function(
             DotPlotAtomic(datas[[nm]], lollipop = TRUE,
                 x = x, y = y, x_sep = NULL, y_sep = y_sep, flip = flip,
                 size_by = size_by, fill_by = fill_by, fill_cutoff = fill_cutoff, fill_reverse = fill_reverse,
-                theme = theme, theme_args = theme_args, palette = palette, palcolor = palcolor, alpha = alpha,
+                theme = theme, theme_args = theme_args, palette = palette[[nm]], palcolor = palcolor[[nm]], alpha = alpha,
                 facet_by = facet_by, facet_scales = facet_scales, facet_ncol = facet_ncol, facet_nrow = facet_nrow, facet_byrow = facet_byrow,
                 x_text_angle = x_text_angle, size_name = size_name, fill_name = fill_name, fill_cutoff_name = fill_cutoff_name,
                 aspect.ratio = aspect.ratio, legend.position = legend.position, legend.direction = legend.direction,
